/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.charts;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.general.DefaultPieDataset;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.StringTokenizer;

public class PieChart extends BaseChart implements Chart {

  public static final String NAME = "pc";

  private PiePlot plot;
  private DefaultPieDataset dataset;

  protected PieChart() {
    super();
    initChart();
  }

  protected PieChart(int width, int height) {
    super(width, height);
    initChart();
  }

  protected PieChart(String width, String height, String colors) {
    initChart();
    configureDimensions(width, height);
    configureColors(colors);
  }

  private void initChart() {
    configureDataset();
    configurePlot();
    initDefaultColors();
  }

  protected void configureColors(String colors) {
    try {
      if (colors != null && colors.length() > 0) {
        StringTokenizer stringTokenizer = new StringTokenizer(colors, ",");
        int i = 0;
        while (stringTokenizer.hasMoreTokens()) {
          plot.setSectionPaint(Integer.toString(i), Color.decode("0x" + stringTokenizer.nextToken()));
          i++;
        }
      }
    }
    catch (Exception e) {
      initDefaultColors();
    }
  }

  private void initDefaultColors() {
    plot.setSectionPaint("0", SERIE1_COLOR);
    plot.setSectionPaint("1", SERIE2_COLOR);
    plot.setSectionPaint("2", SERIE3_COLOR);
    plot.setSectionPaint("3", SERIE4_COLOR);
    plot.setSectionPaint("4", SERIE5_COLOR);
  }

  private void configureDataset() {
    dataset = new DefaultPieDataset();
  }

  private void configurePlot() {
    plot = new PiePlot();
    plot.setDataset(dataset);
    plot.setOutlineVisible(false);
    plot.setBackgroundAlpha(0.0f);
    plot.setCircular(true);
    plot.setLabelGenerator(null);
    plot.setNoDataMessage("No data available");
    plot.setIgnoreNullValues(true);
    plot.setIgnoreZeroValues(true);
    plot.setShadowPaint(null);
    plot.setLabelLinkMargin(0.0);
  }

  protected BufferedImage getChartImage() throws IOException {
    JFreeChart jfreechart = new JFreeChart(null, TextTitle.DEFAULT_FONT, plot, false);
    configureChart(jfreechart, false);
    return getBufferedImage(jfreechart);
  }

  public void addMeasure(Double value, String series) {
    dataset.setValue(series, value);
  }

  public void addMeasures(String values) {
    if (values != null && values.length() > 0) {
      StringTokenizer st = new StringTokenizer(values, ",");
      int i = 0;
      while (st.hasMoreTokens()) {
        double measure = 0;
        try {
          measure = Double.parseDouble(st.nextToken());
        }
        catch (NumberFormatException e) {
        }
        addMeasure(measure, Integer.toString(i));
        i++;
      }
    }
  }

}